@@ -2,7 +2,7 @@
 
 ##############################################################################
 #
-# Copyright (c) 2003 Zope Corporation and Contributors.
+# Copyright (c) 2003 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -141,7 +141,7 @@ Commands:
     - wall time to verify
     - average miliseconds to verify per object.
 
-$Id: zeoserverlog.py 71258 2006-11-21 22:22:48Z jim $
+$Id: zeoserverlog.py 113732 2010-06-21 15:31:11Z ctheune $
 """
 
 import datetime, sys, re, os
@@ -372,7 +372,7 @@ def minute(f, slice=16, detail=1, summary=1):
     rs = []
     ss = []
     cs = []
-    as = []
+    aborts = []
     ts = []
     cls = []
 
@@ -392,7 +392,7 @@ def minute(f, slice=16, detail=1, summary=1):
                     rs.append(r)
                     ss.append(s)
                     cs.append(c)
-                    as.append(a)
+                    aborts.append(a)
                     ts.append(c+a)
                 mlast = m
                 r = s = c = a = 0
@@ -417,7 +417,7 @@ def minute(f, slice=16, detail=1, summary=1):
         rs.append(r)
         ss.append(s)
         cs.append(c)
-        as.append(a)
+        aborts.append(a)
         ts.append(c+a)
 
     if summary:
@@ -429,7 +429,7 @@ def minute(f, slice=16, detail=1, summary=1):
         print 'Reads:   \t', '\t'.join(map(str,stats( rs)))
         print 'Stores:  \t', '\t'.join(map(str,stats( ss)))
         print 'Commits: \t', '\t'.join(map(str,stats( cs)))
-        print 'Aborts:  \t', '\t'.join(map(str,stats( as)))
+        print 'Aborts:  \t', '\t'.join(map(str,stats( aborts)))
         print 'Trans:   \t', '\t'.join(map(str,stats( ts)))
 
 def stats(s):
