@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -13,7 +13,7 @@
 ##############################################################################
 """ZODB-defined exceptions
 
-$Id: POSException.py 40423 2005-11-30 06:29:00Z fdrake $"""
+$Id: POSException.py 113732 2010-06-21 15:31:11Z ctheune $"""
 
 from ZODB.utils import oid_repr, readable_tid_repr
 
@@ -21,9 +21,23 @@ def _fmt_undo(oid, reason):
     s = reason and (": %s" % reason) or ""
     return "Undo error %s%s" % (oid_repr(oid), s)
 
+def _recon(class_, state):
+    err = class_.__new__(class_)
+    err.__setstate__(state)
+    return err
+_recon.__no_side_effects__ = True
+
 class POSError(StandardError):
     """Persistent object system error."""
 
+    def __reduce__(self):
+        # Cope extra data from internal structures
+        state = self.__dict__.copy()
+        state['message'] = self.message
+        state['args'] = self.args
+
+        return (_recon, (self.__class__, state))
+
 class POSKeyError(KeyError, POSError):
     """Key not found in database."""
 
@@ -92,12 +106,11 @@ class ConflictError(TransactionError):
             # avoid circular import chain
             from ZODB.utils import get_pickle_metadata
             self.class_name = "%s.%s" % get_pickle_metadata(data)
-##        else:
-##            if message != "data read conflict error":
-##                raise RuntimeError
 
         self.serials = serials
 
+        self.data = data
+
     def __str__(self):
         extras = []
         if self.oid:
